#!/usr/bin/python
# -*- encoding: utf-8 -*-

# cteni a zapis casu do RTC obvodu DS1307 


import smbus
import datetime
import os
import sys

adr=0x68    # adresa obvodu


bus = smbus.SMBus(1)  # novejsi varianta RasPi (512MB)
#bus = smbus.SMBus(0)  # starsi varianta RasPi (256MB)

try:
  bus.write_byte_data(adr,0x07,0b00010011)  # Control registr nastavit na generovani 32768Hz na INT vystupu 
except: 
  print "CHYBA - RTC obvod nekomunikuje"
  exit(0)


#=============================================
# podprogram pro zjisteni casu v obvodu RTC a jeho zobrazeni, nebo nastaveni casu v RasPi podle RTC
def rtc_info(nastav_cas=False):

  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)
  print "Systemovy cas v RasPi :" , datcas , "UTC"

  nazevdne = [ '???????', 'pondeli' , 'utery  ' , 'streda ' , 'ctvrtek' , 'patek  ' , 'sobota ' , 'nedele ' ]


  rtc =  bus.read_i2c_block_data(adr,0x00)

  chbit = (rtc[0x00] & 0b10000000) >> 7       # indikator zastaveni hodin ( "0" = bezi ; "1" = stop) - CLOCK HALT

  sek = ((rtc[0x00] & 0b01110000)>>4) *10 + (rtc[0x00] & 0b00001111)   # sekundy
  min = ((rtc[0x01] & 0b01110000)>>4) *10 + (rtc[0x01] & 0b00001111)   # minuty
  hod = ((rtc[0x02] & 0b00110000)>>4) *10 + (rtc[0x02] & 0b00001111)   # hodiny
  dvt = ((rtc[0x03] & 0b00000111))                                     # den v tydnu : 1=Po  2=Ut ..... 7=Ne
  den = ((rtc[0x04] & 0b00110000)>>4) *10 + (rtc[0x04] & 0b00001111)   # dny v mesici
  mes = ((rtc[0x05] & 0b00010000)>>4) *10 + (rtc[0x05] & 0b00001111)   # mesic

  rok = ((rtc[0x06] & 0b11110000)>>4) *10 + (rtc[0x06] & 0b00001111)   # roky 00 az 99
  rok = 2000 + rok                                                     # k rokum v RTC se pripocitava jeste stoleti 


   
  txtcas = str(hod).rjust(2,"0") + ":" + str(min).rjust(2,"0") + ":"  + str(sek).rjust(2,"0")
  txtdat = str(nazevdne[dvt]) + "  " +  str(rok) + "-" + str(mes).rjust(2,"0") + "-" + str(den).rjust(2,"0")   
  if (chbit == 1):
    print "POZOR!  Indikator zastaveni hodin je aktivni."

  print "Cas v RTC:     " + txtdat + " " + txtcas + " UTC" 

  print "------ Obsah cele RAM (vcetne casovych registru RTC) ------"
  for adrram in range(64):
    if (adrram < 8) :
      text = "RTC"
    else:
      text = "RAM"
    print text + "[" + str(adrram).rjust(2,"0") + "]=" + str( bus.read_byte_data(adr,adrram)).rjust(3,"0") + "\t",
    if (adrram%4 == 3): print ""
  
  if (nastav_cas == True):  # pokud je parametr tohoto podprogramu True, nastavi se RasPi podle RTC
    print "Cas v RasPi byl nastaven na:"
    prikaz="sudo date -u " + (str(mes).rjust(2,"0") +
                              str(den).rjust(2,"0") +
                              str(hod).rjust(2,"0") + 
                              str(min).rjust(2,"0") + 
                              str(rok) + "." + 
                              str(sek).rjust(2,"0"))
    os.system (prikaz)







#=============================================
# podprogram pro ulozeni systemoveho casu z RasPi do RTC obvodu
def rtc_zapis():
  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)


  # sekundy - registr 0x00
  sekhi = int(datcas.second / 10)
  seklo = datcas.second - (10 * sekhi)
  bus.write_byte_data(adr,0x00,(sekhi*16) + seklo)
  
  # minuty - registr 0x01
  minhi = int(datcas.minute / 10)
  minlo = datcas.minute - (10 * minhi)
  bus.write_byte_data(adr,0x01,(minhi*16) + minlo)
  
  # hodiny - registr 0x02
  hodhi = int(datcas.hour / 10)
  hodlo = datcas.hour - (10 * hodhi)
  bus.write_byte_data(adr,0x02,(hodhi*16) + hodlo)

  # den v tydnu - registr 0x03
  dvt = datetime.datetime.utcnow().isoweekday()  # Po=1 Ut=2 ....Ne=7
  bus.write_byte_data(adr,0x03,dvt)
  
  # den v mesici - registr 0x04
  denhi = int(datcas.day / 10)
  denlo = datcas.day - (10 * denhi)
  bus.write_byte_data(adr,0x04,(denhi*16) + denlo)

  # mesic - registr 0x05
  meshi = int((datcas.month) / 10)
  meslo = datcas.month - (10 * meshi)
  bus.write_byte_data(adr,0x05, (meshi*16) + meslo) 

  # rok - registr 0x06
  rokhi = int(((datcas.year) - 2000) / 10)
  roklo = (datcas.year - 2000) - (10 * rokhi)
  bus.write_byte_data(adr,0x06, (rokhi*16) + roklo) 

  print "Systemovy cas v RasPi :" , datcas , "UTC"
  print "Systemovy cas byl prekopirovan do RTC obvodu."

 

#=============================================
# podprogram pro nastaveni casu v RasPi podle NTP serveru na internetu 
def ntp():   

  print "probiha restart NTP"
  prikaz="sudo /etc/init.d/ntp restart"
  os.system (prikaz)                         # spusteni prikazu v BASHi


#=============================================
# podprogram pro cteni hodnoty z pametove bunky v RTC obvodu 
def ramread(adrram): 
  if (adrram < 0x40):
    valram = bus.read_byte_data(adr,adrram)  
    print "adresa: " , adrram , " obsahuje hodnotu: " , valram
  else:
    print "mimo rozsah"
 

#=============================================
# podprogram pro zapis hodnoty do pametove bunky v RTC obvodu 
def ramwrite(adrram,valram): 
  if (adrram < 0x40 and valram <= 0xff):
    bus.write_byte_data(adr,adrram,valram)  
    print "na adresu: " , adrram , " byla zapsana hodnota: " , valram
  else:
    print "mimo rozsah"



#=============================================
#  ***** hlavni program *****
#=============================================



# vyhodnoceni predavaneho parametru pri spousteni programu
try:                      # odchytavani chyby, ktera by vznikla pri chybejicim parametru
  akce = sys.argv[1]      # parametr spousteneho programu do promenne "akce"
except:                   # kdyz parametr chybi, je nahrazen prazdnym retezcem
  akce=""

akce = akce.lower()       # prevod parametru na mala pismena

if (akce == "-i"):           # informace o aktualnim case v RasPi i v RTC obvodu
  rtc_info()
  
elif (akce == "-pi2rtc"):    # cas z RasPi zapis do RTC obvodu
  rtc_zapis()

elif (akce == "-rtc2pi"):    # nastav cas na RasPi podle RTC ovbodu
  rtc_info(True)

elif (akce == "-ntp2pi"):    # serid cas v RasPi podle internetu
  ntp()

elif (akce == "-ramw"):       # zapis hodnoty do RAM
  try:                        # odchytavani chyby, ktera by vznikla pri chybejicich parametrech
    adrram = int(sys.argv[2]) # adresa pametove bunky
    valram = int(sys.argv[3]) # hodnota pro ulozeni do pametove bunky
  except:                     # kdyz parametr chybi, je nahrazen prazdnym retezcem
    print "Spatne zadane parametry"
    exit()

  ramwrite(adrram,valram)

elif (akce == "-ramr"):       # zapis hodnoty do RAM
  try:                        # odchytavani chyby, ktera by vznikla pri chybejicich parametrech
    adrram = int(sys.argv[2]) # adresa pametove bunky
  except:                     # kdyz parametr chybi, je nahrazen prazdnym retezcem
    print "Spatne zadane parametry"
    exit()

  ramread(adrram)




else:                              # pri jakemkoli jinem parametru se zobrazi napoveda
  print "Pripustne parametry:"
  print "... -i        INFO - Jen zobrazi cas v RasPi a v RTC"
  print "... -pi2rtc   Zapise cas z RasPi do RTC"
  print "... -rtc2pi   Zapise cas z RTC do RasPi"
  print "... -ntp2pi   Restartuje NTP - seridi cas v RasPi podle internetu"
  print "  "
  print "... -ramw ADR VAL   Zapise do pameti RTC obvodu na adresu ADR hodnotu VAL"
  print "... -ramr ADR       Zobrazi hodnotu pameti RTC obvodu na adrese ADR"
  print "                     maximalni mozna adresa je 63, maximalni hodnota je 255"
  print "                        zadavani cisel pouze v desitkove soustave"



